@@ -212,26 +212,37 @@ libpq = ffi.verify('''
         ''', 
         libraries=['pq'],
         library_dirs=[
+            # RHEL, CentOS, Fedora, etc.
             '/usr/pgsql-9.1/lib/',
+            '/usr/pgsql-9.2/lib/',
             ],
         include_dirs=[
+            # Ubuntu, Debian
             '/usr/include/postgresql/', 
+            # RHEL, CentOS, Fedora, etc.
             '/usr/pgsql-9.1/include/',
+            '/usr/pgsql-9.2/include/',
             ],
         ext_package='psycopg2cffi')
 
 
-# copied from ctypes libpq.py
+# imported from postgres/src/include/postgres_ext.h
 
-libpq.PG_DIAG_SEVERITY = ord('S')
-libpq.PG_DIAG_SQLSTATE = ord('C')
+libpq.PG_DIAG_SEVERITY        = ord('S')
+libpq.PG_DIAG_SQLSTATE        = ord('C')
 libpq.PG_DIAG_MESSAGE_PRIMARY = ord('M')
-libpq.PG_DIAG_MESSAGE_DETAIL = ord('D')
-libpq.PG_DIAG_MESSAGE_HINT = ord('H')
-libpq.PG_DIAG_STATEMENT_POSITION = 'P'
-libpq.PG_DIAG_INTERNAL_POSITION = 'p'
-libpq.PG_DIAG_INTERNAL_QUERY = ord('q')
-libpq.PG_DIAG_CONTEXT = ord('W')
-libpq.PG_DIAG_SOURCE_FILE = ord('F')
-libpq.DIAG_SOURCE_LINE = ord('L')
+libpq.PG_DIAG_MESSAGE_DETAIL  = ord('D')
+libpq.PG_DIAG_MESSAGE_HINT    = ord('H')
+libpq.PG_DIAG_STATEMENT_POSITION = ord('P')
+libpq.PG_DIAG_INTERNAL_POSITION = ord('p')
+libpq.PG_DIAG_INTERNAL_QUERY  = ord('q')
+libpq.PG_DIAG_CONTEXT         = ord('W')
+libpq.PG_DIAG_SCHEMA_NAME     = ord('s')
+libpq.PG_DIAG_TABLE_NAME      = ord('t')
+libpq.PG_DIAG_COLUMN_NAME     = ord('c')
+libpq.PG_DIAG_DATATYPE_NAME   = ord('d')
+libpq.PG_DIAG_CONSTRAINT_NAME = ord('n')
+libpq.PG_DIAG_SOURCE_FILE     = ord('F')
+libpq.PG_DIAG_SOURCE_LINE     = ord('L')
 libpq.PG_DIAG_SOURCE_FUNCTION = ord('R')
+
